# Test PostgreSQL TDE/EE upgrades

use strict;
use lib 't';
use TestLib;
use PgCommon;

use Test::More tests => (@MAJORS == 1) ? 1 : 2 + 21 * 3;

if (@MAJORS == 1) {
    pass 'only one major version installed, skipping upgrade tests';
    exit 0;
}

my ($oldv, $newv) = ($MAJORS[0], $MAJORS[-1]);

$ENV{LC_MESSAGES} = 'C';
delete $ENV{LC_ALL};

open my $fh, ">", "/usr/local/bin/tde_encryption_key.sh";
print $fh "#!/bin/sh\n";
print $fh "echo d3b07384d113edec49eaa6238ad5ff00\n";
chmod 0755, $fh;
close $fh;
is_program_out 0, "/usr/local/bin/tde_encryption_key.sh", 0,
    "d3b07384d113edec49eaa6238ad5ff00\n";

foreach my $upgrade_options ('-m dump', '-m upgrade', '-m upgrade --link') {
    next if ($ENV{UPGRADE_METHOD} and $upgrade_options !~ /$ENV{UPGRADE_METHOD}$/); # hack to ease debugging individual methods
    note ("upgrade method \"$upgrade_options\", $MAJORS[0] -> $MAJORS[-1]");

    like_program_out 'root', "pg_createcluster $oldv ee --start -- -K /usr/local/bin/tde_encryption_key.sh", 0,
        qr/Data encryption is enabled/,
        "pg_createcluster $oldv ee with encryption";
    program_ok 'postgres', 'psql -c "create table foo as select 123"', 0;

    like_program_out 'root', "pg_upgradecluster $upgrade_options -v $newv $oldv ee", 0,
        qr/Success. Please check/;
    like_program_out 'root', 'pg_lsclusters -h', 0,
        qr/$oldv ee 5433 down.*\n$newv ee 5432 online/,
        "Upgraded cluster running on correct port";

    is_program_out 'postgres', 'psql -XAtc "select * from foo"', 0, "123\n";
    is_program_out 'postgres', "psql -XAtc 'show data_encryption'", 0, "on\n",
        "New cluster has encryption enabled";

    program_ok 0, "pg_dropcluster $oldv ee --stop", 0;
    program_ok 0, "pg_dropcluster $newv ee --stop", 0;
    system "rm -rf /var/log/postgresql/pg_upgradecluster-$oldv-$newv-ee.*";
    check_clean;
}

unlink "/usr/local/bin/tde_encryption_key.sh";

# vim: filetype=perl
